﻿init python:
    renpy.music.register_channel("sfx1", "sfx", True)
    def find_coordinations(x, y):
        x = x*4
        y = y*4
        return ((x - y) * 32, ((x + y) * 32)//2)
    class iso_player:
        def __init__(self, name, image, throws):
            self.name = name
            self.image = image
            self.x = 0
            self.y = 0
            self.camera_x = 0
            self.camera_y = 0
            self.collection = {
                "iso_heart": 0,
                "iso_sword": 0,
                "iso_time": throws,
            }
            self.bar = chance_bar_class(
                chances = [
                    bar_chance(1, 1, "#ac3232"),
                    bar_chance(2, 1 ,"#306082"),
                    bar_chance(3, 1, "#37946e"),
                    bar_chance(4, 1, "#8f563b"),
                    bar_chance(5, 1, "#76428a"),
                ]
            )
            self.jump_y = 0
        def jump(self):
            self.jump_y = -40
        def land(self):
            self.jump_y = 0

    class iso_tile:
        def __init__(self, tile, icon, x, y, info = ""):
            self.tile = tile
            self.icon = icon
            self.info = info
            self.x = x
            self.y = y

    class iso_board_class:
        def __init__(self, tiles, icons, map, objects, players = None, focus = (0,0)):
            self.tiles = tiles
            self.icons = icons
            self.map = map
            self.objects = objects
            self.players = players or []
            self.focus = focus

            self.camera_x = 0
            self.camera_y = 0
            self.change_focus(*focus)
            self.stage = "start"
            self.moves = 0
            self.playing_index = 0
            self.winner = None
            self.place_players()
        def place_players(self):
            for i in self.players:
                i.x,i.y = self.focus
                i.camera_x, i.camera_y = find_coordinations(*self.focus)
        def remove_player(self, player):
            if not player in self.players:
                return
            self.players.remove(player)
            self.place_players()
        def add_player(self, player):
            if player in self.players:
                return
            self.players.append(player)
            self.place_players()
        def start(self):
            self.stage = "play_dice"
        def throw(self):
            renpy.play("iso board/place.ogg", "sound")
            player = self.players[self.playing_index]
            player.collection["iso_time"] -= 1
            player.bar.reset()
            Show("chance_bar_screen", g = player.bar, gg = self)()
        def thrown(self, moves):
            Hide("chance_bar_screen")()
            self.stage = "move"
            self.moves = moves
        def pass_turn(self):
            renpy.play("iso board/win.ogg", "sound")
            self.stage = "pre_settle"
            for i in self.players:
                if i.collection["iso_time"] > 0:
                    return
            self.stage = "draw"
            
        def next(self):
            self.playing_index += 1
            if self.playing_index > len(self.players)-1:
                self.playing_index = 0
                
            player = self.players[self.playing_index]
            self.change_focus(player.x, player.y)
            self.stage = "play_dice"
        def pre_settle(self):
            self.stage = "pre_settle"
        def settle(self):
            player = self.players[self.playing_index]
            object = self.objects[player.y][player.x]
            if object == 1:
                self.winner = player
                self.stage = "finished"
                renpy.play("iso board/correct.ogg", "sound")
                return
            elif object:
                icon = self.icons[object]
                if icon in ["iso_heart", "iso_sword", "iso_time"]:
                    player.collection[icon] += 1
                    renpy.play("iso board/correct.ogg", "sound")
                elif icon == "iso_skull":
                    player.collection["iso_heart"] -= 1
                    renpy.play("iso board/fail.ogg", "sound")
                elif icon == "iso_star":
                    renpy.play("iso board/correct.ogg", "sound")
                    player.collection["iso_sword"] += 1
                    player.bar.scramble()
                elif icon == "iso_face":
                    renpy.play("iso board/correct.ogg", "sound")
                    player.bar.add_chance(bar_chance(6, 1, "#df7126", 2))
                    player.bar.scramble()
                elif icon == "iso_upgrade":
                    renpy.play("iso board/correct.ogg", "sound")
                    player.bar.add_chance(bar_chance(6, 1, "#323c39", 1))


                self.objects[player.y][player.x] = None
                floating_text(message = None, image = icon)
            self.stage = "changing"
        def move(self, direction):
            if not self.stage == "move":
                return
            if self.moves < 1:
                return
            player = self.players[self.playing_index]
            player.jump()
            renpy.play("iso board/jump.ogg", "sound")
            if direction == "w":
                if player.y > 0 and self.map[player.y - 1][player.x]:
                    player.y -= 1
                    self.moves -= 1
            elif direction == "s":
                if player.y < len(self.map)-1 and self.map[player.y + 1][player.x]:
                    player.y += 1
                    self.moves -= 1
            elif direction == "a":
                if player.x > 0 and self.map[player.y][player.x - 1]:
                    player.x -= 1
                    self.moves -= 1
            elif direction == "d":
                if player.x < len(self.map[0])-1 and self.map[player.y][player.x + 1]:
                    player.x += 1
                    self.moves -= 1

            # renpy.notify(self.map[player.x][player.y])
            player.camera_x, player.camera_y = find_coordinations(player.x, player.y)
            self.change_focus(player.x, player.y)
            if self.moves < 1:
                self.pre_settle()
                # renpy.play("iso board/place.ogg", "sound")
        def change_focus(self, x, y):
            c = find_coordinations(x, y)
            self.camera_x = -c[0]
            self.camera_y = -c[1]

screen iso_board_screen(g):
    modal True
    fixed fit_first True:
        at iso_board_camera(g.camera_x, g.camera_y)
        align .5,.5
        fixed fit_first True:
            at zoom(4, True)
            align .5,.5
            for y,j in enumerate(g.map):
                for x,i in enumerate(j):
                    if i:
                        button:
                            offset ((x - y) * 32), ((x + y) * 32)//2
                            fixed fit_first True:
                                add g.tiles[i]
                                # text "[x],[y]" size 10 align .5,.5
                            at iso_board_button
                            # focus_mask True
                            # action Function(g.change_focus, x, y)
        fixed fit_first True:
            at zoom(4, True)
            align .5,.5
            for y,j in enumerate(g.objects):
                for x,i in enumerate(j):
                    if i:
                        fixed fit_first True:
                            offset ((x - y) * 32), ((x + y) * 32)//2
                            add g.icons[i]
        for i in g.players:
            fixed fit_first True:
                align .5,.0
                at iso_player_move(i.camera_x, i.camera_y)
                fixed fit_first True:
                    at iso_player_jump(i.jump_y)
                    add i.image at zoom(4, True)
                # text "[i.x],[i.y]"
            if i.jump_y:
                timer .2 action Function(i.land)
    # Touch
    fixed fit_first True:
        align 0,1.0 offset 128,-64
        at zoom (3, True)
        # button:
        #     padding 0,0 background None
        #     fixed fit_first True:
        #         add "iso_tile_white"
        #         add "iso_place" align .5,.5
        #     action Function(g.settle) keysym "K_SPACE"
        button:
            padding 0,0 background None offset (32, -16)
            fixed fit_first True:
                add "iso_tile_white"
                add "iso_arrow_1" align .5,.5
            action Function(g.move, "w") keysym "w"
        button:
            padding 0,0 background None offset (-32, 16)
            fixed fit_first True:
                add "iso_tile_white"
                add "iso_arrow_4" align .5,.5
            action Function(g.move, "s") keysym "s"

        button:
            padding 0,0 background None offset (-32, -16)
            fixed fit_first True:
                add "iso_tile_white"
                add "iso_arrow_2" align .5,.5
            action Function(g.move, "a") keysym "a"
        button:
            padding 0,0 background None offset (32, 16)
            fixed fit_first True:
                add "iso_tile_white"
                add "iso_arrow_3" align .5,.5
            action Function(g.move, "d") keysym "d"
    # Stats
    hbox:
        spacing 10
        for n,i in enumerate(g.players):
            if n == g.playing_index:
                use iso_board_stats(i, True)
            else:
                use iso_board_stats(i)

    # GUI
    fixed:
        style_prefix "iso_gui"
        fixed:
            at iso_fade
            align 0,1.0 offset 128,-128 xysize 64*3,46*3
            add g.players[g.playing_index].image align .5,.5 at zoom (4, True)
            text str(g.moves) align .5,.5 yoffset -64
            if g.stage == "play_dice":
                if g.players[g.playing_index].collection["iso_time"] < 1:
                    button:
                        align .5,.5 yoffset -64
                        hbox:
                            text "Pass"
                            add "iso_time" at zoom(2,True) align .5,.5
                        action Function(g.pass_turn)
                else:
                    button:
                        align .5,.5 yoffset -64
                        hbox:
                            text "Throw"
                            add "iso_time" at zoom(2,True) align .5,.5
                        action Function(g.throw)
        if g.stage == "start":
            frame:
                at iso_fade(2)
                align .5,.5
                vbox:
                    text "Ready to rock?"
                    button:
                        text "Start"
                        action Function(g.start)
        elif g.stage == "changing":
            timer 1 action Function(g.next)
        elif g.stage == "pre_settle":
            timer .6 action Function(g.settle)
        elif g.stage == "finished":
            frame:
                at iso_fade
                align .5,.5
                vbox:
                    text "[g.winner.name] won this one."
                    button:
                        text "Done"
                        action Return(g.winner)
        elif g.stage == "draw":
            frame:
                at iso_fade
                align .5,.5
                vbox:
                    text "Everybody have ran out of turns."
                    button:
                        text "Done"
                        action Return("draw")
        
screen iso_board_stats(g, selected = False):
    style_prefix "iso_gui"
    frame:
        if selected:
            background Frame("iso_frame_hover", 32, 32)
        hbox:
            spacing 10
            add g.image at zoom(6, True) yoffset -40
            vbox:
                align .5,.5 xoffset -20
                for key,value in g.collection.items():
                    hbox:
                        add key at zoom(3, True)
                        text str(value)
            vbox:
                align .5,.5
                for i in g.bar.chances:
                    button:
                        xysize 64,((g.bar.size//5)*i.chance)
                        background i.color
                        at chance_bar_animation
                        text str(i.name)

screen iso_board_select(g):
    style_prefix "iso_gui"
    vbox:
        align .5,.5 spacing 20
        text "Choose your players."
        hbox:
            spacing 10
            for i in all_iso_players:
                button:
                    fixed fit_first True:
                        add i.image at zoom(8, True)
                        text i.name align .5,.1
                    action Function(g.add_player, i)
        button:
            text "Done"
            if len(g.players):
                action Return()
    hbox:
        align .5,.9 spacing 10
        for i in g.players:
            button:
                fixed fit_first True:
                    add i.image at zoom(4, True)
                    text i.name align .5,.1
                action Function(g.remove_player, i)

screen iso_board_fight(g):
    style_prefix "iso_gui"





style iso_gui_button:
    background Frame("iso_frame_idle", 32, 32)
    hover_background Frame("iso_frame_hover", 32, 32)
    padding (20,20)
    align (0.5, 0.5)
style iso_gui_frame:
    background Frame("iso_frame_idle", 32, 32)
    padding (20,20)
    align (0.5, 0.5)
style iso_gui_text:
    align (0.5, 0.5)

transform iso_fade(t=1):
    alpha 0
    ease t alpha 1
transform iso_player_jump(y):
    ease .2 yoffset y
transform iso_player_move(x, y):
    ease .4 offset(x,y)
transform iso_board_camera(x, y):
    ease 1 offset(x,y)
transform iso_board_button(x, y):
    on idle:
        ease .2 additive 0
    on hover:
        ease .2 additive .3

init:
    image mako = Crop((0,0,16,32), "iso board/sprites.png")
    image sai = Crop((16,0,16,32), "iso board/sprites.png")
    image liz = Crop((32,0,16,32), "iso board/sprites.png")
    image kaga = Crop((48,0,16,32), "iso board/sprites.png")
    image mi = Crop((64,0,16,32), "iso board/sprites.png")

init:
    image iso_tile_gray = Crop((0,64,64,48), "iso board/sprites.png")
    image iso_tile_blue = Crop((64,64,64,48), "iso board/sprites.png")
    image iso_tile_pink = Crop((128,64,64,48), "iso board/sprites.png")
    image iso_tile_green = Crop((192,64,64,48), "iso board/sprites.png")
    image iso_tile_brown = Crop((0,112,64,48), "iso board/sprites.png")
    image iso_tile_black = Crop((64,112,64,48), "iso board/sprites.png")
    image iso_tile_white = Crop((128,112,64,48), "iso board/sprites.png")
    default all_iso_tiles = [
        None,
        "iso_tile_gray",
        "iso_tile_blue",
        "iso_tile_pink",
        "iso_tile_green",
        "iso_tile_brown",
        "iso_tile_black",
        ]

    image iso_start = Crop((0,160,32,16), "iso board/sprites.png")
    image iso_heart = Crop((32,160,32,16), "iso board/sprites.png")
    image iso_skull = Crop((64,160,32,16), "iso board/sprites.png")
    image iso_sword = Crop((96,160,32,16), "iso board/sprites.png")
    image iso_face = Crop((128,160,32,16), "iso board/sprites.png")
    image iso_star = Crop((160,160,32,16), "iso board/sprites.png")
    image iso_time = Crop((192,160,32,16), "iso board/sprites.png")
    image iso_upgrade = Crop((224,160,32,16), "iso board/sprites.png")
    default all_iso_icon = [
        None,
        "iso_start",
        "iso_heart",
        "iso_skull",
        "iso_sword",
        "iso_face",
        "iso_star",
        "iso_time",
        "iso_upgrade",
        ]

    image iso_arrow_1 = Crop((0,176,32,16), "iso board/sprites.png")
    image iso_arrow_2 = Crop((32,176,32,16), "iso board/sprites.png")
    image iso_arrow_3 = Crop((64,176,32,16), "iso board/sprites.png")
    image iso_arrow_4 = Crop((92,176,32,16), "iso board/sprites.png")

    image iso_place = Crop((128,176,32,16), "iso board/sprites.png")

    image iso_frame_hover:
        Crop((0,192,16,16), "iso board/sprites.png")
        zoom 4 nearest True
    image iso_frame_idle:
        Crop((16,192,16,16), "iso board/sprites.png")
        zoom 4 nearest True

init: # define some players
    default mako = iso_player("Mako", "mako", 13)
    default liz =iso_player("Liz", "liz", 10)
    default sai =iso_player("Sai", "sai", 12)
    default kaga =iso_player("Kaga", "kaga", 9)
    default mi =iso_player("Mi", "mi", 15)

    default all_iso_players = [kaga, sai, mako, liz, mi]

image greenish full = "#30702c"
image greenish half = "#30702c99"

label iso_board_example:
    scene black
    show bg park with dissolve
    pause 1
    show mako normal with dissolve
    "Mako" "Am I late?"
    "Liz" "Yes!"
    show liz normal at midright with dissolve
    "Liz" "But it doesn't matter."
    "Liz" "Everybody gather around!"
    show kaga normal at left with dissolve
    show mi normal at right with dissolve
    show sai normal at midleft with dissolve
    "Sai" "Are we doing this?"
    "Liz" "Yes!"
    "Kaga" "Who goes first?"
    "Liz" "Let him choose."
    
    show greenish half with dissolve
    default iso_1 = iso_board_class(
        tiles = all_iso_tiles,
        icons = all_iso_icon,
        map = csv_to_array("iso board/level_1_board.csv", 1),
        objects = csv_to_array("iso board/level_1_objects.csv", -7),
        players = [],
        focus = (3,12)
        )
    call screen iso_board_select(iso_1)
    "Liz" "Alright, on your marks."
    show greenish full with dissolve
    call screen iso_board_screen(iso_1)
    hide greenish with dissolve
    if _return == "Draw":
        "Liz" "Alright, nobody won."
    else:
        "Liz" "[_return.name], won."
    "Liz" "Let's do another one."
    show greenish half with dissolve
    default iso_2 = iso_board_class(
        tiles = all_iso_tiles,
        icons = all_iso_icon,
        map = csv_to_array("iso board/level_2_board.csv", 1),
        objects = csv_to_array("iso board/level_2_objects.csv", -7),
        players = [],
        focus = (2,5)
        )
    call screen iso_board_select(iso_2)
    show greenish full with dissolve
    call screen iso_board_screen(iso_2)
    hide greenish with dissolve
    if _return == "Draw":
        "Liz" "Alright, nobody won."
    else:
        "Liz" "[_return.name], won."



